
import org.enhydra.jdbc.pool.StandardXAPoolDataSource;
import org.enhydra.jdbc.standard.StandardConnectionPoolDataSource;
import org.enhydra.jdbc.standard.StandardXADataSource;
import org.objectweb.jotm.Jotm;
import org.objectweb.jotm.TraceTm;
import org.objectweb.transaction.jta.TMService;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.transaction.UserTransaction;
import java.util.Properties;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Connection;

public class MultipleConnection {
    private static final String SQL_REQUEST = "select id, foo from testdata";
    private static final String SQL_QUERY = "update testdata set foo = ? where id=1";
    private static final String USAGE = "usage: java StandardXAPoolDataSourceExample [database] [commit|rollback] [number]";
    Connection conn;
    String login = null;
    String password = null;
    String url = null;
    String driver = null;
    private TMService jotm;
    private static final String USER_TRANSACTION_JNDI_NAME = "UserTransaction";

    public MultipleConnection(String [] args) throws Exception {
        Properties prop = new Properties();
        String database = args[0];
        try {
            prop.load(ClassLoader.getSystemResourceAsStream(database + ".properties"));
        } catch (Exception e) {

        }

        login = prop.getProperty("login");
        password = prop.getProperty("password");
        url = prop.getProperty("url");
        driver = prop.getProperty("driver");

        int newValue = 0;
        try {
            newValue = Integer.parseInt(args[2]);
        } catch (NumberFormatException e) {
            System.out.println(USAGE);
            System.out.println("[number] has to be an integer\n");
            System.exit(1);
        }
        String completion = args[1];

        // Get a transction manager
        try {
            // creates an instance of JOTM with a local transaction factory which is not bound to a registry
            jotm = new Jotm(true, false);
            TraceTm.configure();
            InitialContext ictx = new InitialContext();
            ictx.rebind(USER_TRANSACTION_JNDI_NAME, jotm.getUserTransaction());
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        UserTransaction utx = null;
        try {
            System.out.println("create initial context");
            Context ictx = new InitialContext();
            System.out.println("lookup UserTransaction at : " + USER_TRANSACTION_JNDI_NAME);
            utx = (UserTransaction) ictx.lookup(USER_TRANSACTION_JNDI_NAME);
        } catch (Exception e) {
            System.out.println("Exception of type :" + e.getClass().getName() + " has been thrown");
            System.out.println("Exception message :" + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
        StandardConnectionPoolDataSource connect = new StandardConnectionPoolDataSource();

        connect.setUrl(url);
        connect.setDriverName(driver);
        connect.setUser(login);
        connect.setPassword(password);

        // second, we create the pool of connection with the previous object
        StandardXAPoolDataSource spds = new StandardXAPoolDataSource(4);

        StandardXADataSource xads = new StandardXADataSource();
        try {
            xads.setDriverName(prop.getProperty("driver"));
            xads.setUrl(prop.getProperty("url"));
            xads.setTransactionManager(jotm.getTransactionManager());
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        spds.setDataSource(xads);

        System.out.println("Print table before work:");
        printTable();

        try {
            System.out.println("begin a transaction");
            utx.begin();

            System.out.println("update the table");
            conn = spds.getConnection(login, password);

            PreparedStatement pstmt = conn.prepareStatement(SQL_QUERY);
            pstmt.setInt(1, newValue);
            pstmt.executeUpdate();

            conn.close();

            conn = spds.getConnection(login, password);

            PreparedStatement pstmt2 = conn.prepareStatement(SQL_QUERY);
            pstmt2.setInt(1, newValue+2);
            pstmt2.executeUpdate();

            conn.close();

            if (completion.equals("commit")) {
                System.out.println("*commit* the transaction");
                utx.commit();
            } else {
                System.out.println("*rollback* the transaction");
                utx.rollback();
            }
        } catch (Exception e) {
            System.out.println("Exception of type :" + e.getClass().getName() + " has been thrown");
            System.out.println("Exception message :" + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }

        System.out.println("Print table after work:");
        printTable();
    }

    public void printTable() {
        try {
            Statement stmt = conn.createStatement();
            ResultSet rset = stmt.executeQuery(SQL_REQUEST);
            int numcols = rset.getMetaData().getColumnCount();
            for (int i = 1; i <= numcols; i++) {
                System.out.print("\t" + rset.getMetaData().getColumnName(i));
            }
            System.out.println();
            while (rset.next()) {
                for (int i = 1; i <= numcols; i++) {
                    System.out.print("\t" + rset.getString(i));
                }
                System.out.println("");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    static public void main(String [] argv) throws Exception{
        MultipleConnection spdse = new MultipleConnection(argv);
        System.exit(1);
    }
}
